##########################
####  Opening the Data ###
##########################

# Create a simple vector (aka array in FIJI)
variable <- c(0,1,1,2,3,5,8,13,21)
variable
is.vector(variable)

# Generate a bit more complex vector
vec <- vector(mode = "logical", length = 10)
vec
is.vector(vec)

# Generate a 2D matrix with 9 values in 3 columns
my_matrix <- matrix(1:9, ncol = 3)
my_matrix
is.vector(my_matrix)
is.matrix(my_matrix)

#Generating an array. Arrays in R, unlike arrays in FIJI, can have multiple dimensions
# Dimensions = 1
arr <- array(1:4, c(8))
arr

# Dimensions = 2
arr <- array(1:4, c(4,2))
arr

# Dimensions = 3
arr <- array(1:4, c(4,2,2))
arr

#Data frames, having column names and row names
df = as.data.frame(my_matrix)
df
d <- data.frame(letters[1:3], 1:3)
colnames(d) <- c("letters", "numbers")
d

# List: can have all types
out_list <- list(vec, my_matrix)
out_list

#Open NO data from FIJI. Or use the menu
Results <- read.csv("/home/dimitri/Desktop/scripts/Results.csv")

##########################
####  Look at the data ##
##########################

summary(Results)

#which type of data is it?
is.vector(Results)
is.matrix(Results)
is.array(Results)
is.data.frame(Results)

#Extract a column
Results$Major
Results$Feret

#Extract a column and save it separately
feret <- Results$Feret

#Get the summary of 1 column
summary(Results$Feret)

#Extract a value from a column
length(Results$Feret)
Results$Feret[1436]

####################################################
####  Organize your data / extract parts of it  ####
####################################################


#Get the summary of 1 column
summary(Results$Feret)

#Extract a result of the summary
# The output of summary is also a variable (a special class called "SummaryDefault")
# Key is that we can extract values from it
# this object is a table
MedianSummary <- summary(Results$Feret)[3]
is.table(MedianSummary)
MedianSummary[1]
MedianSummary[[1]] 
## The column items can be accessed using Double brackets [[]]

# combined into one command:
summary(Results$Feret)[[3]]

#Length = number of observations in a column
length(Results$Feret)

# Filter for column length
#-------------------------
particles=1000
# use a:b to set the range (first and last). <- to copy the data to a new variable 
feret <- Results$Feret[0:particles]
length(feret)
minferet <- Results$MinFeret[0:particles]

# Make a new dataframe with the extracted column (1 or more columns)
NewResults <- data.frame(feret, minferet)
length(NewResults$feret)

# Filter for particle size
#--------------------------

# Filter according to value / size
sizeLim <- 60
# Report all the rows (of all columns) where the Feret is less than the limit
FilteredResults <- Results[Results$Feret < sizeLim,]


#Take the first x from the size-filtered dataset
particles = 1000 #length(FilteredResults$Label)
Filteredferet <- FilteredResults$Feret[0:particles]
Filteredminferet <- FilteredResults$MinFeret[0:particles]

#Combine into a new Data frame and assure the column names are correct
NewResultsFiltered <- data.frame(Filteredferet, Filteredminferet)
colnames(NewResultsFiltered) <- c("Feret", "MinFeret")

##########################
####  PLOT YOUR DATA  ####
##########################

# Plot a histogram of the feret 
par(mfrow=c(1,1))
hist(NewResults$feret)

# Plot two histograms, comparing the effect of the sizeLimit
par(mfrow=c(1,2))
hist(NewResultsFiltered$Feret, breaks = 50)
hist(Results$Feret, breaks = 50)

# Adjust the number of bins using a simple square root 
par(mfrow=c(1,1))
sqrt(length(NewResultsFiltered$Feret))
# breaks = number of bins. The SQRT of the number of observatons is a good number of bins.
hist(NewResultsFiltered$Feret, breaks = sqrt(length(NewResultsFiltered$Feret)))


# Add titles and color
#---------------------

# prepare a title
Title <- paste("Objects smaller than ", sizeLim, " nm (feret)", "\n", "n = ", particles, "NPs")

# Add the title, some color and Compare different bin sizes in 4 plots
par(mfrow=c(2,2))
hist(NewResultsFiltered$Feret, breaks = 10, main = "10 bins")
hist(NewResultsFiltered$Feret, breaks = 50, main = paste(Title, "\n", "50 bins"), col ="darkgreen")
hist(NewResultsFiltered$Feret, breaks = 250, main = paste(Title, "\n", "250 bins"), ylab = "# particles", 
     xlab = "Size in nm", 
     col ="purple3", 
     border="rosybrown1",
)
hist(NewResultsFiltered$Feret, breaks = 1000, 
     main = paste(Title, "\n", "1000 bins") , 
     ylab = "Density (%)", 
     xlab = "Size in nm", 
     col ="skyblue2",
     border="skyblue",
     prob = TRUE,
     xlim = c(0,sizeLim))

#List of color names
colors()

# A note on Breaks (bins): Freeman-Diaconis rule
#----------------------------------------------

# The bin width is given by F-D as:
# 2 times the interquartal range (= between 3rd quartile and 1st quartile)
# divided by the cube of the number of observations

#https://en.wikipedia.org/wiki/Freedman%E2%80%93Diaconis_rule
#Freedman Diaconis: will calculate the bin width

# to get the bin width, get the IQR, multiply it by 2 and divide by the cube of number of observations. Round it to an integer
# to get the number of bins, get the entire range (max-min) and divide by the value above.

#Calculate range and interquartile range (IQR)
Range = summary(NewResultsFiltered$Feret)[[6]] - summary(NewResultsFiltered$Feret)[[1]]
Range
IQR = summary(NewResultsFiltered$Feret)[[5]] - summary(NewResultsFiltered$Feret)[[2]]
IQR

FreedmanDiaconisBins = round(Range / (2* IQR / (length(NewResultsFiltered$Feret))^(1/3)),0)

#using the Square root approximation:
SqrtObservationsBins = round(sqrt(length(NewResultsFiltered$Feret)),0)

AdditionalTitle <- c("Manually set to","Manually set to", "Square root rule:", "Freedman-Diaconis rule:")

#Make a vector containing the number of bins used by F-D and Square Root rule
breaksapplied <- c(5,10,SqrtObservationsBins, FreedmanDiaconisBins)

par(mfrow=c(2,2))
hist(NewResultsFiltered$Feret, breaks = breaksapplied[1], main = paste(Title, "\n", AdditionalTitle[1], breaksapplied[1], "bins"))
hist(NewResultsFiltered$Feret, breaks = breaksapplied[2], main = paste(Title, "\n", AdditionalTitle[2], breaksapplied[2],"bins"), col ="darkgreen")
hist(NewResultsFiltered$Feret, breaks = breaksapplied[3], main = paste(Title, "\n",AdditionalTitle[3], breaksapplied[3], "bins"), ylab = "# particles", 
     xlab = "Size in nm", 
     col ="red", 
     border="darkred",
)
hist(NewResultsFiltered$Feret, breaks = breaksapplied[4], 
     main = paste(Title, "\n", AdditionalTitle[4],breaksapplied[4],"bins"),
     ylab = "Density (%)", 
     xlab = "Size in nm", 
     col ="blue",
     border="darkblue",
     prob = TRUE,
     xlim = c(0,sizeLim))



# publisheable histogram and density plots
# ----------------------------------------
# Kernel density plots: distribution without needing to care for a bin size
# https://en.wikipedia.org/wiki/Kernel_density_estimation

par(mfrow=c(1,1))
hist(NewResultsFiltered$Feret, breaks=FreedmanDiaconisBins,
     prob = TRUE, # show densities instead of frequencies
     main = paste(Title, "\n", round(breaksapplied[4],0), " bins, with kernel density plot and legend") , 
     ylab = "Density (%)", 
     xlab = "Size in nm", 
     col ="yellow",
     border="orange",
     xlim = c(0,sizeLim),
     ylim = c(0,0.15))
lines(density(NewResultsFiltered$Feret), # density plot
      lwd = 3, 
      col = "chocolate"
      )
legend("topright", inset=.02,legend=c("Histogram", "Kernel Density"), col = c("yellow", "chocolate"),
       lwd = c(10, 3))

#############################################
####  EXTEND THE COMMANDS WITH PACKAGES  ####
#############################################

# Kernel density and local maxima
densityplot <- density(NewResultsFiltered$Feret)


#install.packages("remotes")
#remotes::install_github("rogerswt/wadeTools")
library("wadeTools")




# this library provides find.local.maxima
Threshold = 0.10
Local_maxima <- find.local.maxima(densityplot, thresh = Threshold)
Local_maxima
Local_maxima[[1]]
length(Local_maxima[[1]])


par(mfrow=c(1,1))
plot(densityplot, lty=1, 
     bty="n",
     ylim = c(0,1.1*summary(densityplot$y)[[6]]), 
     main = paste("Kernel density estimate", "\n", length(Local_maxima[[1]]) , " local maxima found", "\n", "Threshold: ", 100*Threshold, "%"),
     xlab = "NP Diameter in nm",
     lwd =3, col = "red")
for (maxima in 1:length(Local_maxima[[1]])   ) {
  x <- c(Local_maxima[[1]][maxima],Local_maxima[[1]][maxima])
  y <- c(0,Local_maxima[[2]][maxima])
  lines(x, y, type = "l", lty = 2)  
  text(Local_maxima[[1]][maxima], Local_maxima[[2]][maxima]+0.01, label = paste(round(Local_maxima[[1]][maxima], 2), "nm"))
} 

#########################
####  Do statistics  ####
#########################

#Normality test
# Simulate a normal distribution with number of overvations same as our variable
normalDist <- rnorm(500, mean=0, sd=1)
histogram(normalDist, breaks = sqrt(length(normalDist)))
shapiro.test(normalDist)

shapiro.test(NewResultsFiltered$Feret)

# H0=Normal distribution 
# p values < 0.05, assume a non-normal distribution 

#####  Many tests, including the one sample Z test, T test and ANOVA assume normality. 
#####  You may still be able to run these tests if your sample size is large enough 
#####  (usually over 20 items). 

#student's test' between MinFeret and Feret
#If significantly different, the particles are not circular
t.test(NewResultsFiltered$Feret, NewResultsFiltered$MinFeret)

# Summary statistics (e.g. mean, standard deviation).
# Two-sample differences tests (e.g. t-test).
# Non-parametric tests (e.g. U-test).
# Matched pairs tests (e.g. Wilcoxon).
# Association tests (e.g. Chi squared).
# Goodness of Fit tests.
# ...


####################################
####  PLOT YOUR DATA: BOXPLOTS  ####
####################################

# simple boxplots
par(mfrow=c(1,1))
boxplot(NewResultsFiltered$Feret,NewResultsFiltered$MinFeret)

# nicer boxplots
boxplot(NewResultsFiltered$Feret,NewResultsFiltered$MinFeret, outline=FALSE,
        main = "Feret and minFeret of AuNP",
        ylab = "Diameter (nm)",
        xlab = "AuNP diameter measure",
        col =c("aliceblue", "bisque"),
        frame = FALSE)
axis(1, 1:2, c("Feret diameter", "Minimal feret diameter"))
legend("topright", c(paste("Mean: ", round(mean(feret),2), " +/- ", round(sd(feret),2), "nm"),
                     paste("Mean: ", round(mean(minferet),2), " +/- ", round(sd(minferet),2), "nm"),
                     paste("T test (p-value): ", signif(t.test(feret,minferet)$p.value, digits=3))),
       inset = 0.01, 
       text.col = c("aliceblue", "bisque", "white"),
       bg = "gray")


###############################
####  REGRESSION ANALYSIS  ####
###############################

model <- lm(Results$Feret~Results$MinFeret) #Create the linear regression
summary(model) #Review the results

#The variable linReg is itself a list
model$coefficients
# first 10 residuals
model$residual[0:10]
#Confidence intervals
confint(model)

#Plot the regression
plot(feret~minferet, pch = 19, col = "blue",
     frame = FALSE,
     ylab ="Feret diameter (nm)",
     xlab = "Minimal feret diameter (nm)",
     xlim = c(0,60),
     ylim = c(0,80),
     main = "linear regression between min and max feret") 
abline(model, lwd=3, col ="red") #Add a regression line


##########################################
####  PLOT YOUR DATA, LOOKING BETTER  ####
##########################################

library("ggplot2")
library("tidyverse")

ggplot(NewResultsFiltered, aes(x = Feret, y = MinFeret)) +  geom_point()

p <- ggplot(Results, aes(x = Feret, y = MinFeret))  
p + geom_point()

p + geom_point() + geom_rug()

p + geom_bin2d(bins = 20)

p + stat_bin_hex()

p + geom_density_2d(bins=25)

library(data.table)
FeretList <- data.frame(NewResultsFiltered$Feret, rep("Feret",length(NewResultsFiltered$Feret)))
colnames(FeretList) <- c("Size", "Name")
MinFeretList <- data.frame(NewResultsFiltered$MinFeret, rep("MinFeret",length(NewResultsFiltered$MinFeret)))
colnames(MinFeretList) <- c("Size", "Name")
NpsList <- rbind(FeretList, MinFeretList)

e <- ggplot(NpsList, aes(y = Size, x = Name))
e + geom_boxplot(outlier.shape = NA) 

